﻿// 
// Copyright (c) 2013 Jason Bell
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
// 

using System;

using Gharen.GLCore;

namespace ExampleDraw
{
    class BoxProgram
        : ShaderProgram
    {
        int PVWLocation;
        int normalLocation;
        int depthBiasLocation;
        int textureEnabledLocation;

        public BoxProgram(GLES30 gl)
            : base(gl, 
                new Shader(gl, gl.GL_VERTEX_SHADER, "Content/boxvertex.glsl"), 
                new Shader(gl, gl.GL_FRAGMENT_SHADER, "Content/boxfragment.glsl"))
        {
            gl.BindAttribLocation(this.Reference, 0, "in_position");
            gl.BindAttribLocation(this.Reference, 1, "in_texcoord");
            gl.BindAttribLocation(this.Reference, 2, "in_normal");

            this.PVWLocation = gl.GetUniformLocation(this.Reference, "PVWMatrix");
            this.textureEnabledLocation = gl.GetUniformLocation(this.Reference, "TextureEnabled");
            this.normalLocation = gl.GetUniformLocation(this.Reference, "NormalMatrix");
            this.depthBiasLocation = gl.GetUniformLocation(this.Reference, "DepthBias");

            Bind();

            gl.Uniform(gl.GetUniformLocation(this.Reference, "Texture"), 0);
            gl.Uniform(gl.GetUniformLocation(this.Reference, "ShadowTexture"), 1);
        }

        public Matrix PVWMatrix
        {
            set
            {
                gl.Uniform(this.PVWLocation, false, ref value);
            }
        }

        public Matrix DepthBias
        {
            set
            {
                gl.Uniform(this.depthBiasLocation, false, ref value);
            }
        }

        public Matrix NormalMatrix
        {
            set
            {
                gl.Uniform(this.normalLocation, false, ref value);
            }
        }

        public bool TextureEnabled
        {
            set
            {
                if (value)
                    gl.Uniform(this.textureEnabledLocation, 1);
                else
                    gl.Uniform(this.textureEnabledLocation, 0);
            }
        }
    }
}
