﻿// 
// Copyright (c) 2013 Jason Bell
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
// 

using System;

using Gharen.GLCore;

namespace ExampleDraw
{
    public class Buffer<T>
        : IDisposable
        where T : struct
    {
        GLES30 gl;

        public int Reference { get; private set; }
        public int BufferTarget { get; private set; }

        public Buffer(GLES30 gl, int bufferTarget, T[] data, int elementSize)
        {
            this.gl = gl;

            this.Reference = gl.GenBuffer();

            this.BufferTarget = bufferTarget;

            gl.BindBuffer(this.BufferTarget, this.Reference);

            gl.BufferData<T>(this.BufferTarget, data.Length * elementSize, data, gl.GL_DYNAMIC_DRAW);

            gl.BindBuffer(this.BufferTarget, 0);
        }

        public void Dispose()
        {
            if (this.Reference != 0)
            {
                gl.DeleteBuffer(this.Reference);
                this.Reference = 0;
            }
        }

        public void Bind()
        {
            gl.BindBuffer(this.BufferTarget, this.Reference);
        }
    }
}
