﻿// 
// Copyright (c) 2013 Jason Bell
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
// 

using System;
using System.IO;

using Gharen.GLCore;

namespace ExampleDraw
{
    class Shader
        : IDisposable
    {
        public int Reference { get; private set; }

        GLES30 gl;

        public Shader(GLES30 gl, int shaderType, string sourceFile)
        {
            this.gl = gl;

            this.Reference = gl.CreateShader(shaderType);

            gl.ShaderSource(this.Reference, ReadSourceFile(sourceFile));

            gl.CompileShader(this.Reference);

            Console.WriteLine(gl.GetShaderInfoLog(this.Reference));
        }

        string ReadSourceFile(string sourceFile)
        {
            if (!File.Exists(sourceFile))
                throw new FileNotFoundException(sourceFile);

            return File.ReadAllText(sourceFile);
        }

        public void Dispose()
        {
            if (this.Reference != 0)
            {
                this.gl.DeleteShader(this.Reference);
                this.Reference = 0;
            }
        }
    }
}
