﻿// 
// Copyright (c) 2013 Jason Bell
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
// 

using System;

using Gharen.GLCore;

namespace ExampleDraw
{
    class ShaderProgram
        : IDisposable
    {
        public int Reference { get; private set; }

        protected GLES30 gl;

        Shader vertexShader;
        Shader fragmentShader;

        public ShaderProgram(GLES30 gl, Shader vertexShader, Shader fragmentShader)
        {
            this.gl = gl;

            this.vertexShader = vertexShader;
            this.fragmentShader = fragmentShader;

            this.Reference = gl.CreateProgram();

            gl.AttachShader(this.Reference, vertexShader.Reference);

            gl.AttachShader(this.Reference, fragmentShader.Reference);

            gl.LinkProgram(this.Reference);

            Console.WriteLine(gl.GetProgramInfoLog(this.Reference));

            gl.ValidateProgram(this.Reference);
        }

        public void Dispose()
        {
            if (this.Reference != 0)
            {
                gl.DeleteProgram(this.Reference);
                this.Reference = 0;
            }
        }

        public void DetachShaders()
        {
            gl.DetachShader(this.Reference, this.vertexShader.Reference);
            gl.DetachShader(this.Reference, this.fragmentShader.Reference);
        }

        public void Bind()
        {
            gl.UseProgram(this.Reference);
        }
    }
}
