﻿// 
// Copyright (c) 2013 Jason Bell
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
// 

using System;

using Gharen.GLCore;

namespace ExampleDraw
{
    class ShadowProgram
        : ShaderProgram
    {

        int projectionLocation;
        int worldLocation;
        int viewLocation;

        public ShadowProgram(GLES30 gl)
            : base(gl,
                new Shader(gl, gl.GL_VERTEX_SHADER, "Content/shadowvertex.glsl"),
                new Shader(gl, gl.GL_FRAGMENT_SHADER, "Content/shadowfragment.glsl"))
        {
            gl.BindAttribLocation(this.Reference, 0, "in_position");

            this.projectionLocation = gl.GetUniformLocation(this.Reference, "Projection");
            this.viewLocation = gl.GetUniformLocation(this.Reference, "View");
            this.worldLocation = gl.GetUniformLocation(this.Reference, "World");
        }

        public Matrix Projection
        {
            set
            {
                gl.Uniform(this.projectionLocation, false, ref value);
            }
        }

        public Matrix World
        {
            set
            {
                gl.Uniform(this.worldLocation, false, ref value);
            }
        }

        public Matrix View
        {
            set
            {
                gl.Uniform(this.viewLocation, false, ref value);
            }
        }
    }
}
