﻿// 
// Copyright (c) 2013 Jason Bell
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
// 

using System;

using Gharen.GLCore;

namespace ExampleDraw
{
    public class VertexArray
        : IDisposable
    {
        public int Reference { get; private set; }

        GLES30 gl;

        public VertexArray(GLES30 gl, Buffer<Vertex> buffer)
        {
            this.gl = gl;

            this.Reference = gl.GenVertexArray();

            gl.BindVertexArray(this.Reference);

            buffer.Bind();

            // Setup position

            gl.EnableVertexAttribArray(0);

            gl.VertexAttribPointer(0,
                3,
                gl.GL_FLOAT,
                false,
                Vertex.SizeInBytes,
                0);

            // Setup texcoords

            gl.EnableVertexAttribArray(1);

            gl.VertexAttribPointer(1,
                2,
                gl.GL_FLOAT,
                false, Vertex.SizeInBytes,
                Vector3.SizeInBytes);

            // Setup normals

            gl.EnableVertexAttribArray(2);

            gl.VertexAttribPointer(2,
                3,
                gl.GL_FLOAT,
                false,
                Vertex.SizeInBytes,
                Vector3.SizeInBytes + Vector2.SizeInBytes);

            gl.BindVertexArray(0);
        }

        public void Dispose()
        {
            if (this.Reference != 0)
            {
                gl.DeleteVertexArray(this.Reference);
                this.Reference = 0;
            }
        }

        public void Bind()
        {
            gl.BindVertexArray(this.Reference);
        }
    }
}
